@@ -12,7 +12,7 @@
  *  Lesser General Public License for more details.
  *
  *===================================================================
- * modified by fduncanh 2021-23
+ * modified by fduncanh 2021-25
  */
 
 #ifndef RAOP_H
@@ -23,11 +23,9 @@
 #include "raop_ntp.h"
 #include "airplay_video.h"
 
-#if defined (WIN32) && defined(DLL_EXPORT)
-# define RAOP_API __declspec(dllexport)
-#else
-# define RAOP_API
-#endif
+#define RAOP_API
+#define MAX_AIRPLAY_VIDEO 10
+#define MIN_STORED_AIRPLAY_VIDEO_DURATION_SECONDS 90   //dont store advertisement playlists
 
 #ifdef __cplusplus
 extern "C" {
@@ -43,6 +41,8 @@ typedef struct playback_info_s {
     uint32_t stallcount;
     double duration;
     double position;
+    double seek_start;
+    double seek_duration;
     float rate;
     bool ready_to_play;
     bool playback_buffer_empty;
@@ -60,6 +60,13 @@ typedef enum video_codec_e {
     VIDEO_CODEC_H265
 } video_codec_t;
 
+typedef enum reset_type_e {
+  RESET_TYPE_NOHOLD,
+  RESET_TYPE_RTP_SHUTDOWN,
+  RESET_TYPE_HLS_SHUTDOWN,
+  RESET_TYPE_HLS_EOS
+} reset_type_t;
+
 struct raop_callbacks_s {
     void* cls;
 
@@ -69,7 +76,7 @@ struct raop_callbacks_s {
     void  (*video_resume)(void *cls);
     void  (*conn_feedback) (void *cls);
     void  (*conn_reset) (void *cls, int reason);
-    void  (*video_reset) (void *cls);
+    void  (*video_reset) (void *cls, reset_type_t reset_type);
   
   
     /* Optional but recommended callback functions (probably not optional, check this)*/
@@ -82,10 +89,12 @@ struct raop_callbacks_s {
     void  (*audio_set_volume)(void *cls, float volume);
     void  (*audio_set_metadata)(void *cls, const void *buffer, int buflen);
     void  (*audio_set_coverart)(void *cls, const void *buffer, int buflen);
+    void  (*audio_stop_coverart_rendering) (void* cls);
     void  (*audio_remote_control_id)(void *cls, const char *dacp_id, const char *active_remote_header);
-    void  (*audio_set_progress)(void *cls, unsigned int start, unsigned int curr, unsigned int end);
+    void  (*audio_set_progress)(void *cls, uint32_t *start, uint32_t *curr, uint32_t *end);
     void  (*audio_get_format)(void *cls, unsigned char *ct, unsigned short *spf, bool *usingScreen, bool *isMedia, uint64_t *audioFormat);
     void  (*video_report_size)(void *cls, float *width_source, float *height_source, float *width, float *height);
+    void  (*mirror_video_activity)(void *cls, double *txusage);
     void  (*report_client_request) (void *cls, char *deviceid, char *model, char *name, bool *admit);
     void  (*display_pin) (void *cls, char * pin);
     void  (*register_client) (void *cls, const char *device_id, const char *pk_str, const char *name);
@@ -99,19 +108,18 @@ struct raop_callbacks_s {
     void  (*on_video_rate) (void *cls, const float rate);
     void  (*on_video_stop) (void *cls);
     void  (*on_video_acquire_playback_info) (void *cls, playback_info_t *playback_video);
-  
+    float  (*on_video_playlist_remove) (void *cls);
 };
+
 typedef struct raop_callbacks_s raop_callbacks_t;
 raop_ntp_t *raop_ntp_init(logger_t *logger, raop_callbacks_t *callbacks, const char *remote,
                           int remote_addr_len, unsigned short timing_rport,
                           timing_protocol_t *time_protocol);
 
-int airplay_video_service_init(raop_t *raop, unsigned short port, const char *session_id);
-
-bool register_airplay_video(raop_t *raop, airplay_video_t *airplay_video);
-airplay_video_t *get_airplay_video(raop_t *raop);
-airplay_video_t *deregister_airplay_video(raop_t *raop);
+airplay_video_t *airplay_video_init(raop_t *raop, unsigned short port, const char *lang);
+char *raop_get_lang(raop_t *raop);
 uint64_t get_local_time();
+void raop_handle_eos(raop_t *raop);
 
 RAOP_API raop_t *raop_init(raop_callbacks_t *callbacks);
 RAOP_API int raop_init2(raop_t *raop, int nohold, const char *device_id, const char *keyfile);
@@ -119,6 +127,7 @@ RAOP_API void raop_set_log_level(raop_t *raop, int level);
 RAOP_API void raop_set_log_callback(raop_t *raop, raop_log_callback_t callback, void *cls);
 RAOP_API int raop_set_plist(raop_t *raop, const char *plist_item, const int value);
 RAOP_API void raop_set_port(raop_t *raop, unsigned short port);
+RAOP_API void raop_set_lang(raop_t *raop, const char *lang);
 RAOP_API void raop_set_udp_ports(raop_t *raop, unsigned short port[3]);
 RAOP_API void raop_set_tcp_ports(raop_t *raop, unsigned short port[2]);
 RAOP_API unsigned short raop_get_port(raop_t *raop);
@@ -129,8 +138,10 @@ RAOP_API void raop_stop_httpd(raop_t *raop);
 RAOP_API void raop_set_dnssd(raop_t *raop, dnssd_t *dnssd);
 RAOP_API void raop_destroy(raop_t *raop);
 RAOP_API void raop_remove_known_connections(raop_t * raop);
-RAOP_API void raop_destroy_airplay_video(raop_t *raop);
-
+RAOP_API void raop_remove_hls_connections(raop_t * raop);
+RAOP_API void raop_destroy_airplay_video(raop_t *raop, int id);
+RAOP_API void raop_playlist_remove(raop_t *raop, void *airplay_video, float position);
+  
 #ifdef __cplusplus
 }
 #endif
