@@ -114,9 +114,8 @@ struct raop_rtp_mirror_s {
 static int
 raop_rtp_mirror_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const char *remote, int remotelen)
 {
-    int family;
-    int ret;
     assert(raop_rtp_mirror);
+    int family = AF_UNSPEC;
     if (remotelen == 4) {
         family = AF_INET;
     } else if (remotelen == 16) {
@@ -125,7 +124,7 @@ raop_rtp_mirror_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const char *rem
         return -1;
     }
     logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror parse remote ip = %s", remote);
-    ret = netutils_parse_address(family, remote,
+    int ret = netutils_parse_address(family, remote,
                                  &raop_rtp_mirror->remote_saddr,
                                  sizeof(raop_rtp_mirror->remote_saddr));
     if (ret < 0) {
@@ -139,12 +138,10 @@ raop_rtp_mirror_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const char *rem
 raop_rtp_mirror_t *raop_rtp_mirror_init(logger_t *logger, raop_callbacks_t *callbacks, raop_ntp_t *ntp,
                                         const char *remote, int remotelen, const unsigned char *aeskey)
 {
-    raop_rtp_mirror_t *raop_rtp_mirror;
-
     assert(logger);
     assert(callbacks);
 
-    raop_rtp_mirror = calloc(1, sizeof(raop_rtp_mirror_t));
+    raop_rtp_mirror_t *raop_rtp_mirror = calloc(1, sizeof(raop_rtp_mirror_t));
     if (!raop_rtp_mirror) {
         return NULL;
     }
@@ -186,8 +183,7 @@ raop_rtp_mirror_thread(void *arg)
     assert(raop_rtp_mirror);
 
     int stream_fd = -1;
-    unsigned char packet[128];
-    memset(packet, 0 , 128);
+    unsigned char packet[128] = {0};
     unsigned char* sps_pps = NULL;
     bool prepend_sps_pps = false;
     int sps_pps_len = 0;
@@ -212,7 +208,6 @@ raop_rtp_mirror_thread(void *arg)
     while (1) {
         fd_set rfds;
         struct timeval tv;
-        int nfds, ret;
         MUTEX_LOCK(raop_rtp_mirror->run_mutex);
         if (!raop_rtp_mirror->running) {
             MUTEX_UNLOCK(raop_rtp_mirror->run_mutex);
@@ -227,6 +222,7 @@ raop_rtp_mirror_thread(void *arg)
 
         /* Get the correct nfds value and set rfds */
         FD_ZERO(&rfds);
+        int nfds = 0;
         if (stream_fd == -1) {
             FD_SET(raop_rtp_mirror->mirror_data_sock, &rfds);
             nfds = raop_rtp_mirror->mirror_data_sock+1;
@@ -234,7 +230,7 @@ raop_rtp_mirror_thread(void *arg)
             FD_SET(stream_fd, &rfds);
             nfds = stream_fd+1;
         }
-        ret = select(nfds, &rfds, NULL, NULL, &tv);
+        int ret = select(nfds, &rfds, NULL, NULL, &tv);
         if (ret == 0) {
             /* Timeout happened */
             continue;
@@ -249,7 +245,7 @@ raop_rtp_mirror_thread(void *arg)
 	    (raop_rtp_mirror && raop_rtp_mirror->mirror_data_sock >= 0) &&
             FD_ISSET(raop_rtp_mirror->mirror_data_sock, &rfds)) {
             struct sockaddr_storage saddr;
-            socklen_t saddrlen;
+            socklen_t saddrlen = 0;
             logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror accepting client");
             saddrlen = sizeof(saddr);
             stream_fd = accept(raop_rtp_mirror->mirror_data_sock, (struct sockaddr *)&saddr, &saddrlen);
@@ -271,8 +267,7 @@ raop_rtp_mirror_thread(void *arg)
                 break;
             }
 
-            int option;
-            option = 1;
+            int option = 1;
             if (setsockopt(stream_fd, SOL_SOCKET, SO_KEEPALIVE, CAST &option, sizeof(option)) < 0) {
                 int sock_err = SOCKET_GET_ERROR();
                 logger_log(raop_rtp_mirror->logger, LOGGER_WARNING,
@@ -330,13 +325,13 @@ raop_rtp_mirror_thread(void *arg)
             /*packet[0:3] contains the payload size */
             int payload_size = byteutils_get_int(packet, 0);
             char packet_description[13] = {0};
-	    char *p = packet_description;
+            char *p = packet_description;
             int n = sizeof(packet_description);
-	    for (int i = 4; i < 8; i++) {
+            for (int i = 4; i < 8; i++) {
                 snprintf(p, n, "%2.2x ", (unsigned int) packet[i]);
                 n -= 3;
                 p += 3;
-	    }
+            }
             ntp_timestamp_raw = byteutils_get_long(packet, 8);
             ntp_timestamp_remote = raop_ntp_timestamp_to_nano_seconds(ntp_timestamp_raw, false);
             if (first_packet) {
@@ -345,14 +340,14 @@ raop_rtp_mirror_thread(void *arg)
                 first_packet = false;
             }
 
-	    /* packet[4] + packet[5] identify the payload type:   values seen are:               *
+            /* packet[4] + packet[5] identify the payload type:   values seen are:               *
              * 0x00 0x00: encrypted packet containing a non-IDR  type 1 VCL NAL unit             *
              * 0x00 0x10: encrypted packet containing an IDR type 5 VCL NAL unit                 *
              * 0x01 0x00: unencrypted packet containing a type 7 SPS NAL + a type 8 PPS NAL unit *
              * 0x02 0x00: unencrypted packet (old protocol) no payload, sent once every second   *
              * 0x05 0x00  unencrypted packet with a "streaming report", sent once per second.    */
 
-	    /* packet[6] + packet[7] may list a payload "option":    values seen are:            *
+            /* packet[6] + packet[7] may list a payload "option":    values seen are:            *
              * 0x00 0x00 : encrypted and "streaming report" packets                              *
              * 0x1e 0x00 : old protocol (seen in AirMyPC) no-payload once-per-second packets     *
              * 0x16 0x01 : seen in most unencrypted h264  SPS+PPS packets                        *
@@ -376,7 +371,7 @@ raop_rtp_mirror_thread(void *arg)
                 readstart = 0;
             }
 
-            while (readstart < payload_size) {
+            while ((int) readstart < payload_size) {
                 // Payload data
                 unsigned char *pos = payload + readstart;
                 ret = recv(stream_fd, CAST pos, payload_size - readstart, 0);
@@ -395,7 +390,7 @@ raop_rtp_mirror_thread(void *arg)
                 break;
             }
 
-	    switch (packet[4]) {
+            switch (packet[4]) {
             case  0x00:
                 // Normal video data (VCL NAL)
 
@@ -409,13 +404,13 @@ raop_rtp_mirror_thread(void *arg)
                     uint64_t ntp_now = raop_ntp_get_local_time();
                     int64_t latency = (ntp_timestamp_local ? ((int64_t) ntp_now) - ((int64_t) ntp_timestamp_local) : 0);
                     logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG,
-                               "raop_rtp video: now = %8.6f, ntp = %8.6f, latency = %9.6f, ts = %8.6f, %s %s",
+                               "raop_rtp video: now = %8.6f, ntp = %8.6f, latency = %9.6f, ts = %8.6f, %s %s, size: %d",
                                (double) ntp_now / SEC, (double) ntp_timestamp_local / SEC, (double) latency / SEC,
-                               (double) ntp_timestamp_remote / SEC, packet_description, h265_video ? h265 : h264);
+                               (double) ntp_timestamp_remote / SEC, packet_description, (h265_video ? h265 : h264), payload_size);
                 }
 
                 unsigned char* payload_out;
-		unsigned char* payload_decrypted;
+                unsigned char* payload_decrypted;
                 /*
                  * nal_types:1   Coded non-partitioned slice of a non-IDR picture
                  *           5   Coded non-partitioned slice of an IDR picture
@@ -437,22 +432,26 @@ raop_rtp_mirror_thread(void *arg)
                                    "raop_rtp_mirror: prepended sps_pps timestamp does not match timestamp of "
                                    "video payload\n%llu\n%llu , discarding", ntp_timestamp_raw, ntp_timestamp_nal);
                         free (sps_pps);
-		        sps_pps = NULL;
+                        sps_pps = NULL;
                         prepend_sps_pps = false;
                 }
 		
                 if (prepend_sps_pps) {
                     assert(sps_pps);
-                    payload_out = (unsigned char*)  malloc(payload_size + sps_pps_len);
+                    payload_out = (unsigned char*) malloc(payload_size + sps_pps_len);
+                    if (!payload_out) {
+                        printf("Memory allocation failed (payload_out)\n");
+                        exit(1);
+                    }
                     payload_decrypted = payload_out + sps_pps_len;
                     memcpy(payload_out, sps_pps, sps_pps_len);
                     free (sps_pps);
-		    sps_pps = NULL;
+                    sps_pps = NULL;
                 } else {
                     payload_out = (unsigned char*)  malloc(payload_size);
                     payload_decrypted = payload_out;
                 }
-                // Decrypt data
+                // Decrypt data: AES-CTR encryption/decryption  does not change the size of the data
                 mirror_buffer_decrypt(raop_rtp_mirror->buffer, payload, payload_decrypted, payload_size);
 
                 // It seems the AirPlay protocol prepends NALs with their size, which we're replacing with the 4-byte
@@ -474,11 +473,11 @@ raop_rtp_mirror_thread(void *arg)
                         valid_data = false;
                         break;
                     }
-		    int nalu_type;
-		    if (h265_video) {
+                    int nalu_type = 0;
+                    if (h265_video) {
                         nalu_type = payload_decrypted[nalu_size] & 0x7e >> 1;;
                         //logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG," h265 video, NALU type %d, size %d", nalu_type, nc_len);
-		    } else {
+                    } else {
                         nalu_type = payload_decrypted[nalu_size] & 0x1f;
                         int ref_idc = (payload_decrypted[nalu_size] >> 5);
                         switch (nalu_type) {
@@ -486,7 +485,7 @@ raop_rtp_mirror_thread(void *arg)
                         case 5:   /*IDR, slice_layer_without_partitioning */
                         case 1:   /*non-IDR, slice_layer_without_partitioning */
                             break;
-	                case 2:   /* slice data partition A */
+                        case 2:   /* slice data partition A */
                         case 3:   /* slice data partition B */
                         case 4:   /* slice data partition C */
                             logger_log(raop_rtp_mirror->logger, LOGGER_INFO,
@@ -526,9 +525,9 @@ raop_rtp_mirror_thread(void *arg)
                                        "unexpected non-VCL NAL unit: nalu_type = %d, ref_idc = %d, nalu_size = %d,"
                                        "processed bytes %d, payloadsize = %d nalus_count = %d",
                                        nalu_type, ref_idc, nc_len, nalu_size, payload_size, nalus_count);
-			    break;
-		        }
-		    }
+                             break;
+                        }
+                    }
                     nalu_size += nc_len;
                 }
                 if (nalu_size != payload_size) valid_data = false;
@@ -540,7 +539,7 @@ raop_rtp_mirror_thread(void *arg)
 		
                 payload_decrypted = NULL;
                 video_decode_struct video_data;
-		video_data.is_h265 = h265_video;
+                video_data.is_h265 = h265_video;
                 video_data.ntp_time_local = ntp_timestamp_local;
                 video_data.ntp_time_remote = ntp_timestamp_remote;
                 video_data.nal_count = nalus_count;   /*nal_count will be the number of nal units in the packet */
@@ -571,13 +570,13 @@ raop_rtp_mirror_thread(void *arg)
                    bytes 56-59 width 
                    bytes 60-63 height 
                    bytes 64-127 all 0x0 
-		*/
+                */
 	      
                 // The information in the payload contains an SPS and a PPS NAL
                 // The sps_pps is not encrypted
                 logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "\nReceived unencrypted codec packet from client:"
                            " payload_size %d header %s ts_client = %8.6f",
-			   payload_size, packet_description, (double) ntp_timestamp_remote / SEC);
+                           payload_size, packet_description, (double) ntp_timestamp_remote / SEC);
 
                 if (packet[6] == 0x56 || packet[6] == 0x5e) {
                     logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "This packet indicates video stream is stopping");
@@ -614,7 +613,7 @@ raop_rtp_mirror_thread(void *arg)
                 logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror width_source = %f height_source = %f width = %f height = %f",
                            width_source, height_source, width, height);
 
-		if (payload_size == 0) {
+                if (payload_size == 0) {
                     logger_log(raop_rtp_mirror->logger, LOGGER_ERR, "raop_rtp_mirror: received type 0x01 packet with no payload:\n"
                                "this indicates non-h264 video but Airplay features bit 42 (SupportsScreenMultiCodec) is not set\n"
                                "use startup option \"-h265\" to set this bit and support h265 (4K) video");
@@ -625,7 +624,7 @@ raop_rtp_mirror_thread(void *arg)
                     free(sps_pps);
                     sps_pps = NULL;
                 }
-		/* test for a H265 VPS/SPS/PPS */
+                /* test for a H265 VPS/SPS/PPS */
                 unsigned char hvc1[] = { 0x68, 0x76, 0x63, 0x31 };
 
                 if (!memcmp(payload + 4, hvc1, 4)) {
@@ -648,12 +647,6 @@ raop_rtp_mirror_thread(void *arg)
                     unsigned char vps_start_code[] = { 0xa0, 0x00, 0x01, 0x00 };
                     unsigned char sps_start_code[] = { 0xa1, 0x00, 0x01, 0x00 };
                     unsigned char pps_start_code[] = { 0xa2, 0x00, 0x01, 0x00 };
-                    unsigned char *vps;
-                    short vps_size;
-                    unsigned char *sps;
-                    short sps_size;
-                    unsigned char *pps;
-                    short pps_size;
 
                     unsigned char * ptr = payload + 0x75;
  
@@ -662,9 +655,9 @@ raop_rtp_mirror_thread(void *arg)
                         raop_rtp_mirror->callbacks.video_pause(raop_rtp_mirror->callbacks.cls);
                         break;
                     }
-                    vps_size = byteutils_get_short_be(ptr, 3);
+                    short vps_size = byteutils_get_short_be(ptr, 3);
                     ptr += 5;
-                    vps = ptr;
+                    unsigned char *vps = ptr;
                     if (logger_debug) {
                         char *str = utils_data_to_string(vps, vps_size, 16);
                         logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "h265 vps size %d\n%s",vps_size, str);
@@ -676,9 +669,9 @@ raop_rtp_mirror_thread(void *arg)
                         raop_rtp_mirror->callbacks.video_pause(raop_rtp_mirror->callbacks.cls);
                         break;
                     }
-                    sps_size = byteutils_get_short_be(ptr, 3);
-		    ptr += 5;
-                    sps = ptr;
+                    short sps_size = byteutils_get_short_be(ptr, 3);
+                    ptr += 5;
+                    unsigned char *sps = ptr;
                     if (logger_debug) {
                         char *str = utils_data_to_string(sps, sps_size, 16);
                         logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "h265 sps size %d\n%s",vps_size, str);
@@ -690,12 +683,12 @@ raop_rtp_mirror_thread(void *arg)
                         raop_rtp_mirror->callbacks.video_pause(raop_rtp_mirror->callbacks.cls);
                         break;
                     }
-		    pps_size = byteutils_get_short_be(ptr, 3);
+                    short pps_size = byteutils_get_short_be(ptr, 3);
                     ptr += 5;
-                    pps = ptr;
+                    unsigned char *pps = ptr;
                     if (logger_debug) {
                         char *str = utils_data_to_string(pps, pps_size, 16);
-		        logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "h265 pps size %d\n%s",pps_size, str);
+                        logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "h265 pps size %d\n%s",pps_size, str);
                         free(str);
                     }
 
@@ -788,6 +781,7 @@ raop_rtp_mirror_thread(void *arg)
                 logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "\nReceived old-protocol once-per-second packet from client:"
                            " payload_size %d header %s ts_raw = %llu", payload_size, packet_description, ntp_timestamp_raw);
                 /* "old protocol" (used by AirMyPC), rest of 128-byte  packet is empty  */
+                break;
             case 0x05:
                 logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "\nReceived video streaming performance info packet from client:"
                            " payload_size %d header %s ts_raw = %llu", payload_size, packet_description, ntp_timestamp_raw);
@@ -795,14 +789,14 @@ raop_rtp_mirror_thread(void *arg)
                  * Sometimes (e.g, when the client has a locked screen), there is a 25kB trailer attached to the packet.    *
                  * This 25000 Byte trailer with unidentified content seems to be the same data each time it is sent.        */
 
-                if (payload_size && raop_rtp_mirror->show_client_FPS_data) {
+                if (payload_size) {
                     //char *str = utils_data_to_string(packet, 128, 16);
                     //logger_log(raop_rtp_mirror->logger, LOGGER_WARNING, "type 5 video packet header:\n%s", str);
                     //free (str);
 		    
                     int plist_size = payload_size;
                     if (payload_size > 25000) {
-		        plist_size = payload_size - 25000;
+                        plist_size = payload_size - 25000;
                         if (logger_debug) {
                             char *str = utils_data_to_string(payload + plist_size, 16, 16);
                             logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG,
@@ -811,13 +805,21 @@ raop_rtp_mirror_thread(void *arg)
                         }
                     }
                     if (plist_size) {
-                        char *plist_xml;
-                        uint32_t plist_len;
+                        char *plist_xml = NULL;
+                        uint32_t plist_len = 0;
                         plist_t root_node = NULL;
                         plist_from_bin((char *) payload, plist_size, &root_node);
-                        plist_to_xml(root_node, &plist_xml, &plist_len);
-                        logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "%s", plist_xml);
-                        free(plist_xml);
+                        if (raop_rtp_mirror->callbacks.mirror_video_activity) {
+                            double txusage = 0.0;
+                            plist_t tx_usage_avg_node = plist_dict_get_item(root_node, "txUsageAvg");
+                            plist_get_real_val(tx_usage_avg_node, &txusage);
+                            raop_rtp_mirror->callbacks.mirror_video_activity(raop_rtp_mirror->callbacks.cls, &txusage);
+                        }
+                        if (raop_rtp_mirror->show_client_FPS_data) {
+                            plist_to_xml(root_node, &plist_xml, &plist_len);
+                            logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "%s", plist_xml);
+                            free(plist_xml);
+                        }
                     }
                 }
                 break;
@@ -848,13 +850,13 @@ raop_rtp_mirror_thread(void *arg)
 
     logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror exiting TCP thread");
     if (conn_reset&& raop_rtp_mirror->callbacks.conn_reset) {
-      raop_rtp_mirror->callbacks.conn_reset(raop_rtp_mirror->callbacks.cls, 1);
+        raop_rtp_mirror->callbacks.conn_reset(raop_rtp_mirror->callbacks.cls, 1);
     }
 
     if (unsupported_codec) {
         closesocket(raop_rtp_mirror->mirror_data_sock);
         raop_rtp_mirror_stop(raop_rtp_mirror);
-        raop_rtp_mirror->callbacks.video_reset(raop_rtp_mirror->callbacks.cls);
+        raop_rtp_mirror->callbacks.video_reset(raop_rtp_mirror->callbacks.cls, RESET_TYPE_RTP_SHUTDOWN);
     }
 
     return 0;
