@@ -39,7 +39,7 @@ extern "C" {
 #include <stdio.h>
 
 struct X11_Window_s {
-    Display * display;
+    Display *display;
     Window window;
 } typedef X11_Window_t;
 
@@ -48,23 +48,36 @@ static void get_X11_Display(X11_Window_t * X11, char *display_name) {
     X11->window = (Window) NULL;
 }
 
+//static void free_X11_Display(X11_Window_t *X11) {
+//  if (X11->display) {
+//      XCloseDisplay(X11->display);
+//        X11->display = NULL;
+//	X11->window = (Window) NULL;
+//    }
+//}
+  
 static Window enum_windows(const char * str, Display * display, Window window, int depth) {
-    int i;
-    XTextProperty text;
-    XGetWMName(display, window, &text);
     char* name = NULL;
     XFetchName(display, window, &name);
-    if (name != 0 &&  strcmp(str, name) == 0) {
-        return window;
+    if (name) {
+        if (strcmp(str, name) == 0) {
+            XFree(name);
+            return window;
+        }
+        XFree(name);
     }
+
     Window _root, parent;
     Window* children = NULL;
-    unsigned int n;
+    unsigned int n = 0;
     XQueryTree(display, window, &_root, &parent, &children, &n);
     if (children != NULL) {
-        for (i = 0; i < n; i++) {
+        for (int i = 0; i < n; i++) {
             Window w = enum_windows(str, display, children[i], depth + 1);
-            if (w) return w;
+            if (w) {
+                XFree(children);
+                return w;
+            }
         }
         XFree(children);
     }
